@@ -123,10 +123,10 @@ TEST_CASE("1. argument nargs", "[argument]")
                 = parser.parse_args(_make_vec("--store", new_value, "--append", new_value, "--extend", new_value));
         CHECK(args2.get<std::string>("--store") == new_value);
         CHECK(args2.get<std::string>("--append") == new_value);  // return array value
-        CHECK(args2.get<std::string>("--extend") == new_value);  // return array value
+        CHECK_THROWS(args2.get<std::string>("--extend"));  // return array value
         CHECK(args2.get<std::vector<std::string> >("--store").size() == 1);
         CHECK(args2.get<std::vector<std::string> >("--append").size() == 1);
-        CHECK(args2.get<std::vector<std::string> >("--extend").size() == 1);
+        CHECK(args2.get<std::vector<std::string> >("--extend").size() == 1 * 3);
 
         // override args
         argparse::Namespace args3
@@ -134,13 +134,12 @@ TEST_CASE("1. argument nargs", "[argument]")
                                               "--store", new_value, "--append", new_value, "--extend", new_value));
         CHECK(args3.get<std::vector<std::string> >("--store").size() == 1);
         CHECK(args3.get<std::vector<std::string> >("--append").size() == 2);
-        CHECK(args3.get<std::vector<std::string> >("--extend").size() == 2);
+        CHECK(args3.get<std::vector<std::string> >("--extend").size() == 2 * 3);
     }
 
     SECTION("1.4. nargs ? positional") {
         parser.add_argument("store").action(argparse::store).nargs("?").default_value(default_value);
         parser.add_argument("append").action(argparse::append).nargs("?");
-//        parser.add_argument({ "extend" }).action(argparse::extend).nargs("?");  // invalid in python without arguments
 
         argparse::Namespace args1 = parser.parse_args(_make_vec());
         CHECK(args1.get<std::string>("store") == default_value);
@@ -163,6 +162,48 @@ TEST_CASE("1. argument nargs", "[argument]")
         CHECK_THROWS(parser.parse_args(_make_vec(new_value, new_value, new_value)));
     }
 
+    SECTION("1.4.1. nargs ? positional extend") {
+        parser.add_argument("extend").action(argparse::extend).nargs("?");  // invalid in python without arguments
+
+        CHECK_THROWS(parser.parse_args(_make_vec()));
+
+        argparse::Namespace args1 = parser.parse_args(_make_vec(new_value));
+        CHECK_THROWS(args1.get<std::string>("extend"));  // return array value
+        CHECK(args1.get<std::vector<std::string> >("extend").size() == 3);
+        CHECK(args1.to_string() == "Namespace(extend=['n', 'e', 'w'])");
+
+        CHECK_THROWS(parser.parse_args(_make_vec(new_value, new_value)));
+        CHECK_THROWS(parser.parse_args(_make_vec(new_value, new_value, new_value)));
+    }
+
+    SECTION("1.4.2. nargs ? optional extend") {
+        parser.add_argument("--extend").action(argparse::extend).nargs("?");
+
+        argparse::Namespace args1 = parser.parse_args("--extend " + new_value);
+        CHECK_THROWS(args1.get<std::string>("extend"));  // return array value
+        CHECK(args1.get<std::vector<std::string> >("extend").size() == 3);
+        CHECK(args1.to_string() == "Namespace(extend=['n', 'e', 'w'])");
+
+        argparse::Namespace args2 = parser.parse_args("--extend=" + new_value);
+        CHECK_THROWS(args2.get<std::string>("extend"));  // return array value
+        CHECK(args2.get<std::vector<std::string> >("extend").size() == 3);
+        CHECK(args2.to_string() == "Namespace(extend=['n', 'e', 'w'])");
+    }
+
+    SECTION("1.4.3. nargs ? optional extend with const value") {
+        parser.add_argument("--extend").action(argparse::extend).nargs("?").const_value(new_value);
+
+        argparse::Namespace args0 = parser.parse_args(_make_vec("--extend"));
+        CHECK_THROWS(args0.get<std::string>("extend"));  // return array value
+        CHECK(args0.get<std::vector<std::string> >("extend").size() == 3);
+        CHECK(args0.to_string() == "Namespace(extend=['n', 'e', 'w'])");
+
+        argparse::Namespace args1 = parser.parse_args(_make_vec("--extend", new_value));
+        CHECK_THROWS(args1.get<std::string>("extend"));  // return array value
+        CHECK(args1.get<std::vector<std::string> >("extend").size() == 3);
+        CHECK(args1.to_string() == "Namespace(extend=['n', 'e', 'w'])");
+    }
+
     SECTION("1.5. nargs * optional") {
         parser.add_argument("--store").action(argparse::store).nargs("*").default_value(default_value);
         // default value are invalid in python if flag used
@@ -697,3 +738,58 @@ TEST_CASE("1. argument nargs", "[argument]")
         CHECK(parser.parse_args("a --bar c").to_string() == "Namespace(args=['a', '--bar', 'c'], baz='123', bar=None)");
     }
 }
+
+TEST_CASE("2. combined nargs", "[argument]")
+{
+    argparse::ArgumentParser parser = argparse::ArgumentParser().exit_on_error(false);
+
+    SECTION("2.1. nargs N") {
+        parser.add_argument("-f").nargs(3);
+
+        CHECK_THROWS(parser.parse_args("-f"));
+        CHECK_THROWS(parser.parse_args("-f="));
+        CHECK(parser.parse_args("-f n e w").to_string() == "Namespace(f=['n', 'e', 'w'])");
+    }
+
+    SECTION("2.2. nargs +") {
+        parser.add_argument("-f").nargs("+", 3);
+
+        CHECK_THROWS(parser.parse_args("-f"));
+        CHECK_THROWS(parser.parse_args("-f="));
+        CHECK(parser.parse_args("-f n e w").to_string() == "Namespace(f=['n', 'e', 'w'])");
+    }
+
+    SECTION("2.3. nargs *") {
+        parser.add_argument("-f").nargs("*", 3);
+
+        CHECK(parser.parse_args("-f").to_string()       == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f=").to_string()      == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f n e w").to_string() == "Namespace(f=['n', 'e', 'w'])");
+    }
+
+    SECTION("2.4. nargs ?") {
+        parser.add_argument("-f").nargs("?", 3);
+
+        CHECK(parser.parse_args("-f").to_string()       == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f=").to_string()      == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f n e w").to_string() == "Namespace(f=['n', 'e', 'w'])");
+    }
+
+    SECTION("2.5. nargs ? extend with valid const") {
+        parser.add_argument("-f").action("extend").nargs("?", 3).const_value("con");
+
+        CHECK(parser.parse_args("-f").to_string()       == "Namespace(f=['c', 'o', 'n'])");
+        CHECK(parser.parse_args("-f=").to_string()      == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f=new").to_string()   == "Namespace(f=['n', 'e', 'w'])");
+        CHECK(parser.parse_args("-f new").to_string()   == "Namespace(f=['n', 'e', 'w'])");
+    }
+
+    SECTION("2.6. nargs ? extend with invalid const") {
+        parser.add_argument("-f").action("extend").nargs("?", 3).const_value("const");
+
+        CHECK_THROWS(parser.parse_args("-f"));
+        CHECK(parser.parse_args("-f=").to_string()      == "Namespace(f=[])");
+        CHECK(parser.parse_args("-f=new").to_string()   == "Namespace(f=['n', 'e', 'w'])");
+        CHECK(parser.parse_args("-f new").to_string()   == "Namespace(f=['n', 'e', 'w'])");
+    }
+}
